package org.alef1.bookmark;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Iterator;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.java.plugin.ObjectFactory;
import org.java.plugin.PluginManager;
import org.java.plugin.PluginManager.PluginLocation;
import org.java.plugin.registry.PluginDescriptor;
import org.java.plugin.standard.StandardPluginLocation;

public class Main {

    public static void main(String[] args) {
        new Main().start();
    }
   
    private void start() {

        loadPlugins();
       
        createShell();
        
        addDefaultBookmarks();
		addDynamicBookmarks();
		
        setListeners();
       
        shell.open();
       
        Display display = shell.getDisplay();
       
        while (!shell.isDisposed()) {
           
            if (!display.readAndDispatch())
                display.sleep();
           
        }
       
        display.dispose();
       
    }

    private void loadPlugins() {

    	pluginManager = ObjectFactory.newInstance().createManager();
    	
		File pluginsDir = new File("plugins");
    	
    	File[] plugins = pluginsDir.listFiles(new FilenameFilter() {

			public boolean accept(File dir, String name) {
				return name.toLowerCase().endsWith(".zip");
			}
    		
    	});
    	
		try {
			
			PluginLocation[] locations = new PluginLocation[plugins.length];
			
			for (int i = 0; i < plugins.length; i++) {
				locations[i] = StandardPluginLocation.create(plugins[i]);
			}

			pluginManager.publishPlugins(locations);
			
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
    	
    }

    private void addDefaultBookmarks() {
        
        root = new TreeItem(tree, SWT.NULL);
		root.setText("Bookmarks");
       
        TreeItem item = new TreeItem(root, SWT.NULL);
        item.setText("Google");
        item.setData("http://www.google.com");
       
        TreeItem dev = new TreeItem(root, SWT.NULL);
        dev.setText("Java");
        item = new TreeItem(dev, SWT.NULL);
        item.setText("java.net");
        item.setData("http://www.java.net");
        item = new TreeItem(dev, SWT.NULL);
        item.setText("Java World");
        item.setData("http://www.javaworld.com/");
       
    }

	private void addDynamicBookmarks() {
		
		try {
			
			Iterator it = pluginManager.getRegistry().getPluginDescriptors().iterator();
			
			while (it.hasNext()) {
				
				PluginDescriptor p = (PluginDescriptor) it.next();
				
				BookmarkSection section = (BookmarkSection) pluginManager.getPlugin(p.getId());
				
			    TreeItem entry = new TreeItem(root, SWT.NULL);
			    entry.setText(section.category());
	
			    for (int i = 0; i < section.count(); i++) {
			    	
			        TreeItem item = new TreeItem(entry, SWT.NULL);
			        
			        item.setText(section.getName(i));
			        item.setData(section.getUrl(i));
			        
			    }
			    
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
    
    private void setListeners() {
       
        tree.addSelectionListener(new SelectionListener() {

            public void widgetSelected(SelectionEvent e) {
            }

            public void widgetDefaultSelected(SelectionEvent e) {
               
                TreeItem item = (TreeItem) e.item;
               
                if (item.getItemCount() == 0) {
                   
                    CTabItem tab = new CTabItem(tabFolder, SWT.CLOSE);
                   
                    tab.setText(item.getText());
                   
                    Browser b = new Browser(tabFolder, SWT.BORDER);
                   
                    b.setUrl((String) item.getData());

                    tab.setControl(b);
                   
                    tabFolder.setSelection(tabFolder.getItemCount() - 1);
                   
                }
               
            }
           
        });
       
    }

    private Shell shell = null;  //  @jve:decl-index=0:visual-constraint="153,55"
    private SashForm sashForm = null;
    private Tree tree = null;
    private CTabFolder tabFolder = null;
	private TreeItem root;
	private PluginManager pluginManager;

    /**
     * This method initializes shell   
     *
     */
    private void createShell() {
        shell = new Shell();
        shell.setLayout(new FillLayout());
        shell.setText("Plugin Aware Application");
        createSashForm();
        shell.setSize(new Point(435, 196));
    }

    /**
     * This method initializes sashForm
     *
     */
    private void createSashForm() {
        sashForm = new SashForm(shell, SWT.V_SCROLL);
        sashForm.setOrientation(SWT.HORIZONTAL);
        tree = new Tree(sashForm, SWT.BORDER);
        tree.setItemCount(0);
        createTabFolder();
        sashForm.setWeights(new int[] {30, 70});
    }

    /**
     * This method initializes tabFolder   
     *
     */
    private void createTabFolder() {
       
        tabFolder = new CTabFolder(sashForm, SWT.NONE);
        tabFolder.setSimple(false);
       
        Display display = shell.getDisplay();
       
        tabFolder.setSelectionForeground(display.getSystemColor(SWT.COLOR_WHITE));
        tabFolder.setSelectionBackground(new Color[]{display.getSystemColor(SWT.COLOR_DARK_BLUE),
                display.getSystemColor(SWT.COLOR_BLUE),
                display.getSystemColor(SWT.COLOR_WHITE),
                display.getSystemColor(SWT.COLOR_WHITE)},
                new int[] {25, 50, 100});
       
    }

} 